`define NUM_CORE 2

module oc_avr_hp_cm4_top(handsome_clk, handsome_din_top, handsome_dout_top, cp2clk, cp2clken, ireset);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input cp2clk;
	input cp2clken;
	input ireset;

localparam input_width = 50;
localparam output_width = 62;
localparam reducer_log4 = 3;
localparam reducer_log4_top = 4;

wire [`NUM_CORE-1:0] insert_nop;
wire [`NUM_CORE-1:0] block_irq;
wire [`NUM_CORE-1:0] cpuwait;
wire [`NUM_CORE-1:0] valid_instr;
wire [`NUM_CORE-1:0] change_flow;
wire [`NUM_CORE-1:0] iore;
wire [`NUM_CORE-1:0] iowe;
wire [`NUM_CORE-1:0] ramre;
wire [`NUM_CORE-1:0] ramwe;
wire [`NUM_CORE-1:0] irqack;
wire [`NUM_CORE-1:0] sleepi;
wire [`NUM_CORE-1:0] irqok;
wire [`NUM_CORE-1:0] globint;
wire [`NUM_CORE-1:0] wdri;
wire [15:0] inst [`NUM_CORE-1:0];
wire [7:0] dbusin [`NUM_CORE-1:0];
wire [22:0] irqlines [`NUM_CORE-1:0];
wire [15:0] pc [`NUM_CORE-1:0];
wire [5:0] adr [`NUM_CORE-1:0];
wire [15:0] ramadr [`NUM_CORE-1:0];
wire [7:0] dbusout [`NUM_CORE-1:0];
wire [4:0] irqackad [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({insert_nop[i],block_irq[i],cpuwait[i],inst[i],dbusin[i],irqlines[i]}));

		oc_avr_hp_cm4 oc_stamp(
			.cp2clk(cp2clk),
			.cp2clken(cp2clken),
			.ireset(ireset),
			.insert_nop(insert_nop[i]),
			.block_irq(block_irq[i]),
			.cpuwait(cpuwait[i]),
			.inst(inst[i]),
			.dbusin(dbusin[i]),
			.irqlines(irqlines[i]),
			.valid_instr(valid_instr[i]),
			.change_flow(change_flow[i]),
			.iore(iore[i]),
			.iowe(iowe[i]),
			.ramre(ramre[i]),
			.ramwe(ramwe[i]),
			.irqack(irqack[i]),
			.sleepi(sleepi[i]),
			.irqok(irqok[i]),
			.globint(globint[i]),
			.wdri(wdri[i]),
			.pc(pc[i]),
			.adr(adr[i]),
			.ramadr(ramadr[i]),
			.dbusout(dbusout[i]),
			.irqackad(irqackad[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({valid_instr[i],change_flow[i],iore[i],iowe[i],ramre[i],ramwe[i],irqack[i],sleepi[i],irqok[i],globint[i],wdri[i],pc[i],adr[i],ramadr[i],dbusout[i],irqackad[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
